Level01 = Core.class(Sprite)

function Level01:init()
	-- the bg
	application:setBackgroundColor(0x0000a0)

	-- the plugin
	require "liquidfun"
	--b2.setScale(32) -- gideros default: 30px = 1m

	-- the liquidfun world
	self.world = b2.World.new(0, 24, true) -- heavy gravity!

	-- the layers so we can put mobile control over it
	self.gamelayer = Sprite.new()

	-- liquidfun debug
	local debugdraw = b2.DebugDraw.new()
	self.world:setDebugDraw(debugdraw)
--	self.gamelayer:addChild(debugdraw)

	-- the level
	self.gamelayer:addChild(Level01_Shape.new(self.world))

	-- the player
	self.player = Player.new(self.world, "gfx/player/HQ_Trooper_all.png", 6, 13)
	self.player.lives = 3
	self.player.body:setPosition(4 * 32, 2 * 32) -- position the body, the bitmap will follow in enterframe
	self.gamelayer:addChild(self.player)

	-- some enemies
	self.enemy = LF_Dynamic_Contour.new(self.world, "gfx/nmes/ghost.png", 1, 1, true, 1, 0.5, 0.2)
	self.enemy.body:setPosition(10 * 32, 0) -- position the body, the bitmap will follow in enterframe
	self.gamelayer:addChild(self.enemy)
	self.enemy02 = LF_Dynamic_Contour.new(self.world, "gfx/nmes/ghost.png", 0.5, 0.5, true, 1, 0.5, 0.2)
	self.enemy02.body:setPosition(20 * 32, 0) -- position the body, the bitmap will follow in enterframe
	self.gamelayer:addChild(self.enemy02)

	-- some platforms
	local posx1, posy1 = 110 * 32, 6 * 32
	local mpf01 = LF_Kinematic_Box.new(self.world, "gfx/platforms/mytile32a.png", 10, 1, true,
			10, 0, 1,
			0, 1, posx1, posy1, -- move vertically
			posx1, posy1 - 2 * 32, posx1, posy1 + 2 * 32,
			11) -- solid platform
	self.gamelayer:addChild(mpf01)
	local posx2, posy2 = 123 * 32, 5 * 32
	local mpf02 = LF_Kinematic_Box.new(self.world, "gfx/platforms/mytile32a.png", 5, 1, true,
			10, 0, 1,
			1, 0, posx2, posy2, -- move horizontally
			posx2 - 4 * 32, posy2, posx2 + 2 * 32, posy2,
			11) -- solid platform
	self.gamelayer:addChild(mpf02)
	local posx3, posy3 = 43 * 32, 3 * 32
	local mpf03 = LF_Kinematic_Box.new(self.world, "gfx/platforms/mytile32b.png", 6, 1, true,
			10, 0, 1,
			0, 0, posx3, posy3,
			posx3 - 2 * 32, posy3, posx3 + 2 * 32, posy3,
			10) -- pass through platform
	self.gamelayer:addChild(mpf03)
	local posx4, posy4 = 10 * 32, 5 * 32
	local mpf04 = LF_Kinematic_Box.new(self.world, "gfx/platforms/mytile32b.png", 4, 1, true,
			10, 0, 1,
			0, 1, posx4, posy4,
			posx4, posy4 - 2 * 32, posx4, posy4 + 2 * 32,
			10) -- pass through platform
	self.gamelayer:addChild(mpf04)

	-- a bridge
	local bridge = Bridge.new(self.world, 90 * 32, 3 * 32)
	self.gamelayer:addChild(bridge)

	-- the game
	self:addChild(self.gamelayer)

	-- listeners
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	self.world:addEventListener(Event.END_CONTACT, self.onEndContact, self)
	self.world:addEventListener(Event.PRE_SOLVE, self.onPreSolveContact, self)
	self.world:addEventListener(Event.POST_SOLVE, self.onPostSolveContact, self)
	self:addEventListener("enterBegin", self.onTransitionInBegin, self)
	self:addEventListener("enterEnd", self.onTransitionInEnd, self)
	self:addEventListener("exitBegin", self.onTransitionOutBegin, self)
	self:addEventListener("exitEnd", self.onTransitionOutEnd, self)
end

-- the game loop
function Level01:onEnterFrame(e)
	local playerposx, playerposy = self.player.body:getPosition()
	-- camera follow player
	local offsetX = -(playerposx - contentwidth / 2)
	self.gamelayer:setX(offsetX)
	local offsetY = -(playerposy - contentheight / 2)
	self.gamelayer:setY(offsetY)

	self.world:step(1/60, 1, 1)
end

-- collisions handler
function Level01:onBeginContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	if fixtureA.type == 2 or fixtureB.type == 2 then
		-- is on floor
		self.player.numfeetcontacts += 1
		self.player.isgoingdownplatform = false
	end
	if fixtureA.type == 11 or fixtureB.type == 11 then
		-- is on platform
		self.player.numplatformcontacts += 1
--		self.player.isgoingdownplatform = false
	end
	if fixtureA.type == 10 or fixtureB.type == 10 then
		-- is on platform
		self.player.numplatformcontacts += 1
--		self.player.isgoingdownplatform = false
	end
end

function Level01:onPreSolveContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	local platform = nil
	if fixtureA.type == 10 then
		platform = fixtureA
	end
	if fixtureB.type == 10 then
		platform = fixtureB
	end
	if platform == nil then return end

	-- going down platform
	if self.player.isgoingdownplatform then
		e.contact:setEnabled(false)
		return
	end
	-- pass through platform
	local velocityx, velocityy = self.player.body:getLinearVelocity()
	local posx, posy = self.player.body:getPosition()
	local pospx, pospy = platform:getBody():getPosition()
	if velocityy > 0 then -- going down = collision
		if self.player.isdown then -- player pressing down
			e.contact:setEnabled(false) -- else going up = no collision
		else
			e.contact:setEnabled(true)
		end
	elseif velocityy < -7 then -- going up = no collision (treshold) -7
		e.contact:setEnabled(false)
	end
end

function Level01:onPostSolveContact(e)
end

function Level01:onEndContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	if fixtureA.type == 2 or fixtureB.type == 2 then
		-- is not on floor
		self.player.numfeetcontacts -= 1
	end
end

-- EVENT LISTENERS
function Level01:onTransitionInBegin()
	self:myKeysPressed()
end

function Level01:onTransitionInEnd()
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	-- mobile controls
	local mymobile = Mobile.new(self.player)
	self:addChild(mymobile)
end

function Level01:onTransitionOutBegin()
	self:removeEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
	self.vPad:free()
end

function Level01:onTransitionOutEnd()
end

-- KEYS HANDLER
function Level01:myKeysPressed()
	self:addEventListener(Event.KEY_DOWN, function(e)
		-- for mobiles and desktops
		if e.keyCode == KeyCode.BACK or e.keyCode == KeyCode.ESC then
--			sceneManager:changeScene("menu", 1, transitions[2], easing.outBack)
		end
	end)
end
