LF_Bullet = Core.class(Sprite)

function LF_Bullet:init(xworld, xtexture, xscalex, xscaley, xfixedrotation,
			xdensity, xrestitution, xfriction,
			xlvx, xlvy, xstartx, xstarty,
			xfixturetype)
	self.world = xworld
	self.removebullet = false

	local texture = Texture.new(xtexture, true, {wrap = Texture.REPEAT})
	self.bitmap = Bitmap.new(texture)
	self.bitmap:setAnchorPoint(0.5, 0.5)
	self.bitmap:setScale(xscalex, xscaley)
	local w, h = self.bitmap:getWidth(), self.bitmap:getHeight()
	self:addChild(self.bitmap)

	self.body = xworld:createBody { type = b2.DYNAMIC_BODY }
	self.body:setBullet(true)
	self.body:setGravityScale(0)
	self.body:setPosition(xstartx, xstarty)
	self.body:setFixedRotation(xfixedrotation)
	self.shape = b2.PolygonShape.new()
	self.shape:setAsBox(self.bitmap:getWidth() / 2, self.bitmap:getHeight() / 2)
	self.fixture = self.body:createFixture {
		shape = self.shape, 
		density = xdensity, restitution = xrestitution, friction = xfriction
	}
	self.fixture.type = xfixturetype or 0
	-- moving or not moving?
	self.lvx, self.lvy = xlvx, xlvy
	self.startx, self.starty = xstartx, xstarty
	self.maxx = math.abs(self.startx + 12 * 32)
	self.body:setLinearVelocity(xlvx or 0, xlvy or 0)

	-- listeners
	xworld:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	xworld:addEventListener(Event.END_CONTACT, self.onEndContact, self)
	xworld:addEventListener(Event.PRE_SOLVE, self.onPreSolveContact, self)
	xworld:addEventListener(Event.POST_SOLVE, self.onPostSolveContact, self)
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

-- game loop
function LF_Bullet:onEnterFrame(dt)
	if self.body ~= nil then
		local tposx, tposy = self.body:getPosition()
		-- remove bullet?
		if self.removebullet or tposx > self.maxx then
			self.world:destroyBody(self.body)
			self.body = nil
			self:getParent():removeChild(self)
		end
		-- bitmap
		self.bitmap:setPosition(tposx, tposy)
	end
--	print(stage:getNumChildren())
end

-- collisions handler
function LF_Bullet:onBeginContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	if fixtureA.type == 100 then
		self.removebullet = true
	end
	if fixtureB.type == 100 then
		self.removebullet = true
	end
end

function LF_Bullet:onPreSolveContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	local platformfixture =  nil
	local otherfixture = nil
	if fixtureA.type == 10 then
		platformfixture = fixtureA
		otherfixture = fixtureB
	elseif fixtureB.type == 10 then
		platformfixture = fixtureB
		otherfixture = fixtureA
	end
	if not platformfixture then return end

	-- pass through platform
	local velocityx, velocityy = otherfixture:getBody():getLinearVelocity()
end

function LF_Bullet:onPostSolveContact(e)
end

function LF_Bullet:onEndContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	if fixtureA.type == 2 or fixtureB.type == 2 then
		-- is not on floor
	end
	if fixtureA.type == 1 or fixtureB.type == 1 then
		-- is not on platform
	end
end
